@@ -482,8 +482,12 @@ class AutoTuner:
                             )
                         except Exception as e:
                             shapes = self._get_input_sizes(tensors)
+                            logger.warning(
+                                f"[Autotuner]: Skipping tactic {r} {tac}, due to failure while profiling."
+                            )
 
-                            logger.error(
+                            # Log stacktrace as debug to not spam log
+                            logger.debug(
                                 f"[Autotuner]: Failed when profiling {r} {tac}, shapes={shapes}. Error occurred: {e}"
                             )
 
