@@ -137,12 +137,11 @@ static auto makeTmaShapeStrideAbc(GemmOptions const& options, int mM, int mN, in
   std::vector<uint64_t> shape = {static_cast<uint64_t>(hiddenSize),
                                  static_cast<uint64_t>(numTokens)};
   if (useTmaOobOpt /* also implies input/output activation */) {
-    // If TMA OOB optimization is used, we use 3D logical shape (M, tileM, K) or (N, tileN, K).
-    // The outer dimension is extended to make room for the possible counterbalance positive
-    // offset from the middle "bound" dimension. The counterbalance should be no more than
-    // ctaTileNumTokens.
+    // If TMA OOB optimization is used:
+    // Shape [hidden, tokens]                      Stride [1, hidden] becomes
+    // Shape [hidden, tileN, TmaDimMax, TmaDimMax] Stride [1, hidden, XLargeN - hidden, hidden]
     shape = {static_cast<uint64_t>(hiddenSize), static_cast<uint64_t>(ctaTileNumTokens),
-             static_cast<uint64_t>(numTokens + ctaTileNumTokens)};
+             static_cast<uint64_t>(tg::TmaDimMax), static_cast<uint64_t>(tg::TmaDimMax)};
   } else if (isWeights) {
     // If the matrix is a weights matrix, we use 3D logical shape (B, M, K) or (B, N, K).
     shape = {static_cast<uint64_t>(hiddenSize), static_cast<uint64_t>(numTokens),
@@ -153,7 +152,8 @@ static auto makeTmaShapeStrideAbc(GemmOptions const& options, int mM, int mN, in
   // Swap the first two dimension as mentioned before.
   std::vector<uint64_t> stride = {1, static_cast<uint64_t>(hiddenSize)};
   if (useTmaOobOpt) {
-    stride = {1, static_cast<uint64_t>(hiddenSize), static_cast<uint64_t>(hiddenSize)};
+    stride = {1, static_cast<uint64_t>(hiddenSize), static_cast<uint64_t>(tg::XLargeN - hiddenSize),
+              static_cast<uint64_t>(hiddenSize)};
   } else if (isWeights) {
     stride = {1, static_cast<uint64_t>(hiddenSize),
               static_cast<uint64_t>(hiddenSize) * static_cast<uint64_t>(numTokens)};
@@ -164,6 +164,10 @@ static auto makeTmaShapeStrideAbc(GemmOptions const& options, int mM, int mN, in
 
   // Alternate layouts (MajorMn and BlockMajorK) do not apply to matrixC
   if (matrixType != MatrixType::MatrixC) {
+    // When using 2CTA MMA, we only need to load half of the tile in each CTA for B.
+    if (matrixType == MatrixType::MatrixB && tileShape[1] > 1 && options.mClusterDimX == 2) {
+      tileShape[1] /= 2;
+    }
     gemm::MatrixLayout layout =
         (matrixType == MatrixType::MatrixA) ? options.mLayoutA : options.mLayoutB;
     // Note, only the weights support non MajorK layouts
@@ -290,6 +294,7 @@ static auto makeTmaShapeStrideSfAb(int mM, int mN, int mK, MatrixType matrixType
   }
   return std::make_tuple(std::vector<uint64_t>{}, std::vector<uint64_t>{}, std::vector<uint32_t>{});
 }
+
 template <class GemmOptions_>
 static KernelParams setKernelParams(
     GemmOptions_ const& options, bool const batchM, void const* ptrA, void const* ptrB, void* ptrC,
@@ -428,7 +433,7 @@ static KernelParams setKernelParams(
       tg::Dtype const dTypeSf =
           (options.mDtypeB == tg::Dtype::E2m1) ? tg::Dtype::E4m3 : tg::Dtype::UE8m0;
 
-      if (batchedGemm::doesRouteImplUseTma(options.mRouteImpl)) {
+      if (batchedGemm::doesRouteImplUseTma(options.mRouteSfsImpl.value())) {
         // The input is NOT padded:
         // [act0, act1, act2, ...]
 
@@ -445,7 +450,7 @@ static KernelParams setKernelParams(
         params.tmaSfB[0] = gemm::buildNdTmaDescriptor(
             dTypeSf, options.mMmaKind, shapeSfB, strideSfB, tileShapesSfB, const_cast<void*>(dSfB),
             /*doSwizzle*/ true);
-      } else if (batchedGemm::doesRouteImplUseNoRoute(options.mRouteImpl)) {
+      } else if (batchedGemm::doesRouteImplUseNoRoute(options.mRouteSfsImpl.value())) {
         // The input is padded:
         // [act0, padding, padding, ... TileN size .., act1, padding, padding, ...]
 
@@ -473,7 +478,6 @@ static KernelParams setKernelParams(
     } else {
       params.ptrC = ptrC;
     }
-
   } else {
     // B is the expert
     if (0 != options.mN % options.mTileN) {
@@ -508,7 +512,7 @@ static KernelParams setKernelParams(
       tg::Dtype const dTypeSf =
           (options.mDtypeA == tg::Dtype::E2m1) ? tg::Dtype::E4m3 : tg::Dtype::UE8m0;
 
-      if (options.mRouteImpl == batchedGemm::RouteImpl::NoRoute) {
+      if (options.mRouteSfsImpl.value() == batchedGemm::RouteImpl::NoRoute) {
         // The input is padded:
         // [act0, padding, padding, ... tileM size .., act1, padding, padding, ...]
         auto const inputNumTokensSfA = ctaOffset * options.mTileM;
